# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from cms.models.pluginmodel import CMSPlugin

from filer.fields.file import FilerFileField
from filer.fields.image import FilerImageField

from jmb.core.db.cmsmodels import StatusModel, WrapModel

from acomea.contact.models import Group
from acomea.event.models import ActiveEvent

###################################
## CMS MODELS                     #
###################################


class TraceRequestsSettings(CMSPlugin, StatusModel, WrapModel):

    is_modal = models.BooleanField(
        _("Modal window"),
        default=True,
        help_text=_(
            "Uncheck this field in "
            "case of non modal form"
        ),
    )
    extra_css = models.TextField(
        verbose_name=_('Extra CSS'),
        null=True, blank=True,
        default=None,
        help_text=_(
            "(Optional) put here extra css rules "
            "(directly insert rules, e.g. color: "
            "color: #666666;)."
            "Rules will be applied to div container."
            " Require wrap content checked"
        )
    )
    image = FilerImageField(
        null=True, blank=True,
        default=None,
        verbose_name=_("Image"),
        related_name="clickable_image",
        help_text=_(
            "(Optional) If populated, the image will "
            "be used as the clickable link "
            "for file's download"
        )
    )
    image_height = models.PositiveIntegerField(
        _("Image height (px)"),
        null=True, blank=True,
        help_text=_("Imgae height in px")
    )
    image_width = models.PositiveIntegerField(
        _("Image width (px)"),
        null=True, blank=True,
        help_text=_("Image width in px")
    )
    message = models.CharField(
        _("Message"),
        max_length=200,
        null=True, blank=True,
        help_text= _("Message shown in the link")
    )
    header_modal_message = models.CharField(
        _("Header modal message"),
        max_length=200,
        null=True, blank=True,
        help_text= _(
            "Message shown in modal's "
            "header (if empty, it will be"
            " used the 'message' field)"
        )
    )
    body_modal_message = models.TextField(
        _("body modal message"),
        max_length=200,
        null=True, blank=True,
        help_text=_(
            "Message shown in modal's header ("
            "if empty, it will be used the "
            "'message' field)"
        )
    )
    additional_modal_css_rules = models.TextField(
        verbose_name=_('Additional modal CSS rules'),
        blank=True,
        default=None,
        help_text=_("(Optional) extra css rules for modal's "
                    "div and its children, separated by '|'.\n"
                    "- rules into {} will be applied directly"
                    " to container div;\n"
                    "- other rules must have child css selector (e.g. 'a {...}'"
                    " will be applied to every a child element"
                    "of modal's div)")
    )
    form = models.CharField(
        _("Form"),
        max_length=150,
        choices=settings.CONTACT_CMSPLUGINS_TRACEREQUESTS_FORM_CLASSES,
        default="acomea.contact.forms.TraceRequestsForm",
        help_text=_("Form that will be used in the modal window")
    )
    form_template_name = models.CharField(
        _("Form template name"),
        max_length=100,
        choices=settings.CONTACT_CMSPLUGINS_TRACEREQUESTS_FORM_TEMPLATES,
        default="contact/cms_plugins/trace_request_form.html"
    )
    done_template_name = models.CharField(
        _("Done template name"),
        max_length=100,
        choices=settings.CONTACT_CMSPLUGINS_TRACEREQUESTS_DONE_TEMPLATES,
        default="contact/cms_plugins/trace_request_done.html"
    )
    contact_group = models.ForeignKey(
        Group,
        related_name="request_trace",
        verbose_name=_("Contact groups"),
        help_text=_(
            "Contact group in which " 
            "requests are saved"
        )
    )
    internal_file = FilerFileField(
        null=True, blank=True,
        default=None,
        verbose_name=_("File"),
        help_text=_("File to download")
    )
    external_file_url = models.URLField(
        _("Alternate file URL"),
        blank=True, null=True,
        help_text=_(
            "(Optional) Alternative file URL. If "
            "populated, prevails over 'File' field ")
    )
    mandatory_amc = models.BooleanField(
        _("Mandatory A.M.C. flag"),
        default=False,
        help_text=_(
            "If checked, 'Autorizzazione materiale commerciale'" 
            " field will be required in the form (to be used with "
            "trace requests form w/ privacy fields)"
        )
    )
    mandatory_amc_err_message = models.CharField(
        _("Mandatory A.M.C. custom error message"),
        max_length=250,
        blank=True,
        help_text=_(
           "(Optional)custom error message for required field "
           "'Autorizzazione materiale commerciale' (if Mandatory "
            "A.M.C. flag is checked)" 
        )
    )
    thanks_text_message = models.TextField(
        _("Thanks text message"),
        blank=True, null=True,
        help_text=_(
            "(Optional) Text that will be "
            "shown in the success modal window"
        )
    )
    success_tracking_url = models.URLField(
        _("Success tracking URL"),
        blank=True, null=True,
        help_text=_("URL for success tracking")
    )
    submit_background_img = FilerImageField(
        null=True, blank=True,
        default=None,
        related_name="sumbit_image",
        verbose_name=_("Submit button image"),
        help_text=_(
            "(Optional) Form submit "
            "button image background"
        )
    )
    submit_background_img_width = models.PositiveIntegerField(
        _("Submit button image width"),
        null=True, blank=True,
        help_text=_(
            "(Optional) Form submit "
            "button image width (in px)"
        )
    )
    submit_background_img_height = models.PositiveIntegerField(
        _("Submit button image height"),
        null=True, blank=True,
        help_text=_(
            "(Optional) Form submit "
            "button image height (in px)"
        )
    )
    submit_alignment = models.CharField(
        _("Submit button alignment"),
        max_length=100,
        null=True, blank=True,
        choices=settings.CONTACT_CMSPLUGINS_TRACEREQUESTS_FORM_SUBMIT_ALIGNMENT_CHOICES,
        help_text=_(
            "(Optional) Form submit button alignment"
        )
    )
    submit_text = models.CharField(
        _("Submit button text"),
        max_length=100,
        null=True, blank=True,
        help_text=_(
            "(Optional) Form submit button text. "
            "To be used with background image"
        )
    )
    event = models.ForeignKey(
        ActiveEvent,
        related_name="trace_requests_plugins",
        null=True, blank=True,
        verbose_name=_("event"),
    )

    def __unicode__(self):
        _unicode = "%s - %s" % (self._meta.verbose_name, self.id)
        if self.status:
            return "On - %s" % _unicode
        else:
            return "Off - %s" % _unicode

    class Meta:
        verbose_name = _("TraceRequest")
        app_label = "contact"


class PrivacyManagementSettings(CMSPlugin, StatusModel, WrapModel):

    modal_header_title =  models.CharField(
        _("Modal Header title"),
        max_length=200,
        default="Gestione privacy",
        help_text="(Optional) modal header title"
    )
    form = models.CharField(
        _("Form"),
        max_length=150,
        choices=settings.CONTACT_CMSPLUGINS_PRIVACYMANAGEMENT_FORM_CLASSES,
        default="acomea.contact.forms.PrivacyManagementForm",
        help_text=_("Form that will be used in the modal window")
    )
    form_template_name = models.CharField(
        _("Form template name"),
        max_length=100,
        choices=settings.CONTACT_CMSPLUGINS_PRIVACYMANAGEMENT_FORM_TEMPLATES,
        default="contact/cms_plugins/privacy_management_form.html"
    )

    def __unicode__(self):
        _unicode = "%s - %s" % (self._meta.verbose_name, self.id)
        if self.status:
            return "On - %s" % _unicode
        else:
            return "Off - %s" % _unicode

    class Meta:
        verbose_name = _("Gestione Privacy")
        app_label = "contact"


class ConfirmContactSettings(CMSPlugin, StatusModel, WrapModel):

    confirmation_done_template = models.CharField(
        _("Confirmation done template name"),
        max_length=150,
        choices=settings.CONTACT_CMSPLUGINS_CONFIRMCONTACT_DONE_TEMPLATES,
        default="contact/cms_plugins/contact_confirm_done.html"
    )
    confirmation_done_msg = models.TextField(
        _("Confirmation done message"),
        default="Inserire un messaggio"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        default="home",
        help_text=_("slug pagina insuccesso")
    )

    def __unicode__(self):
        _unicode = "%s - %s" % (self._meta.verbose_name, self.id)
        if self.status:
            return "On - %s" % _unicode
        else:
            return "Off - %s" % _unicode

    class Meta:
        verbose_name = _("Conferma contatto")
        app_label = "contact"
