"""
views module for cms plugged apps - acomea.contact
"""
# -*- coding: utf-8 -*-
from django.conf import settings
from django.views.generic import FormView, TemplateView
from django.views.generic.edit import ProcessFormView
from django.http import HttpResponse
from django.shortcuts import redirect
from django import forms
from django.shortcuts import render_to_response

from web.utils.generic import get_acomea_host, get_acomea_tcp_protocol

from acomea.contact.models import Contact
from acomea.contact.forms import ConfirmPrivacyForm
from acomea.contact.utils.generic import (
    send_confirm_privacydata_contact, send_delete_contact_notify_to_backoffice,
    check_contact_confirm_url, has_related_user,
    get_new_token_and_cidb36
)

###################################
## CMS APP VIEWS                  #
###################################

class ConfirmPrivacyFormView(FormView):
    """
    main form view for confirm privacy data
    """
    template_name = settings.CONTACT_CMSAPPS_CONFIRMPRIVACYFORM_TEMPLATE
    form_class = ConfirmPrivacyForm
    success_url = "/"

    def __init__(self, **kwargs):
        super(ConfirmPrivacyFormView, self).__init__(**kwargs)
        self.email = None
        self.extra_context = {}


    def get_form(self, form_class):
        return form_class(
            email=self.email
        )
    def get_context_data(self, **kwargs):
        context = super(ConfirmPrivacyFormView, self).get_context_data(**kwargs)
        context.update(self.extra_context)
        return context

    def get(self, request, *args, **kwargs):

        # TOKEN/CIDB CHECK
        if not (request.method == "GET" and (
            'email' in request.GET and 'cidb36' in request.GET and 'token' in request.GET
        )):
            return redirect("/")

        contact = check_contact_confirm_url(request)
        if not contact:
            return redirect("/")

        # CHECK FOR RELATED USER
        if has_related_user(contact):
            self.template_name = settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_HAS_RELATED_USER_TEMPLATE
            self.extra_context.update({
                'has_related_user':True,
                'contact':contact
            })
            return super(ConfirmPrivacyFormView, self).get(request, *args, **kwargs)

        self.email = self.request.GET.get('email', None)
        if not self.email:
            return redirect("/")
        self.extra_context.update({
            'cidb36':request.GET['cidb36'],
            'token':request.GET['token'],
        })
        return super(ConfirmPrivacyFormView, self).get(request, *args, **kwargs)

    def form_valid(self, form):
        contact = form.save()
        confirm_button_value = form.data.get('confirm_button', None)
        if confirm_button_value:
            if confirm_button_value == "confirm":
                # privacy data confirmed
                contact.bonifica = True
                contact.save()
                new_token, cidb36 = get_new_token_and_cidb36(contact)
                send_confirm_privacydata_contact(
                    contact, contact.get_default_email(), request=self.request, data_confirmed=True
                )

                # FIXME: remove hardcoded url
                return redirect(
                    "/registrazione/conferma-privacy/dataconfirmed?cidb36=%s&token=%s" % (
                        cidb36,
                        new_token
                    )
                )
        return super(ConfirmPrivacyFormView, self).form_valid(form)

    def post(self, request, *args, **kwargs):
        form = ConfirmPrivacyForm(request.POST)
        if form.is_valid():
            return self.form_valid(form)
        else:
            self.extra_context.update({'contact_id':request.POST.get('contact_id', None)})
            return self.form_invalid(form)

    def form_invalid(self, form):
        # FIXME: form workaround - to be fixed
        _form = ConfirmPrivacyForm(contact_id=self.extra_context.pop('contact_id'))
        _form._errors = {}
        _form._errors.update(form._errors)
        return self.render_to_response(self.get_context_data(form=_form))


class ConfirmPrivacyDataTemplateView(TemplateView):
    """
    privacy data confirmed view
    """
    template_name = settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_CONFIRMDATA_TEMPLATE

    def __init__(self, **kwargs):
        super(ConfirmPrivacyDataTemplateView, self).__init__(**kwargs)
        self.extra_context = {}

    def get(self, request, *args, **kwargs):

         # TOKEN/CIDB CHECK
        if not (request.method == "GET" and (
            'cidb36' in request.GET and
            'token' in request.GET
        )):
            return redirect("/")

        contact = check_contact_confirm_url(request)
        if not contact:
            return redirect("/")
        self.extra_context.update({ 'contact' : contact})
        contact.auth_trattamento_dati = True
        contact.save() # this make the token valid once
        return self.render_to_response(self.get_context_data())

    def get_context_data(self, **kwargs):
        context = super(ConfirmPrivacyDataTemplateView, self).get_context_data(**kwargs)
        context.update(self.extra_context)
        return context

class CancelContactTemplateView(TemplateView):
    """
    view for contact delete
    """
    template_name = settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_CANCEL_CONTACT_TEMPLATE

    def __init__(self, **kwargs):
        super(CancelContactTemplateView, self).__init__(**kwargs)
        self.extra_context = {}


    def get_context_data(self, **kwargs):
        context = super(CancelContactTemplateView, self).get_context_data(**kwargs)
        context.update(self.extra_context)
        return context

    def get(self, request, *args, **kwargs):
        return redirect("/")

    def post(self, request, *args, **kwargs):
        contact_id = request.POST.get('contact_id', None)
        if contact_id:
            try:
                Contact.objects.get(pk=contact_id)
            except Contact.DoesNotExist:
                redirect("/")

            # TOKEN/CIDB CHECK
            if not (request.method == "POST" and (
                'cidb36' in request.POST and 'token' in request.POST
            )):
                return redirect("/")

            contact = check_contact_confirm_url(request, method='POST')
            if not contact:
                return redirect("/")

            contact_data = {
                'first_name' : contact.first_name,
                'last_name' : contact.last_name,
             }
            self.extra_context.update({'contact_data':contact_data})
            send_delete_contact_notify_to_backoffice(contact)
            contact.delete()

            return self.render_to_response(self.get_context_data())











