# -*- coding: utf-8 -*-
import datetime
import logging

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.auth.models import User

from jmb.core.db.models import DateModel, UserModel, StatusModel, DefaultModel, SEOModel, OrderedModel
from jmb.core.middleware.thread_local import get_current_user, get_request
from jmb.l10n.models import Country
from jmb.mailer.models import EmailObject

from acomea.profilo.models import AcomeaProfilo

privacy_log = logging.getLogger('acomea.privacy')

def register_contact(nome, cognome, email, gruppi, societa=None, cellulare=None, citta=None):
    # stringa, stringa, stringa, qs Group.objects, stringa, stringa, stringa
    # se esiste la mail la mette a default e aggiunge eventuali gruppi non ancora aggiunti
    # altrimenti crea il contatto e gli aggiunge i gruppi
    create = False
    if Email.objects.filter(email__exact=email).count() > 0:
        mail = Email.objects.get(email=email)
        mail.default = True
        mail.save()
        contact = mail.contact
    else:
        contact = Contact(first_name=nome, last_name=cognome)
        contact.save()
        email = Email(contact=contact, email=email, default=True)
        email.save()

        create = True
    for g in gruppi:
        contact.groups.add(g)

    if create:
        if societa:
            contact.organization = societa
        contact.save()
        if cellulare:
            pc, _ = PhoneCategory.objects.get_or_create(name='cellulare')
            Phone.objects.create(contact=contact, phone=cellulare, category=pc)
        if citta:
            #asc, _ = AddressScope.objects.get_or_create(name='default')
            a = Address.objects.create(contact=contact, name='', number='', locality=citta)
    return contact

class Role (DateModel, UserModel, StatusModel):
    """
    Stores contact roles
    EXAMPLE: 'sales manager', 'development manager'
    """
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Contact role")
        verbose_name_plural = _("Contact roles")
        ordering = ('name',)
        permissions = (
            ('can_list_role', 'Can list Role'),
        )

    def __unicode__(self):
        return u"%s" % self.name

class Title (StatusModel, OrderedModel, DateModel, UserModel):
    """
    Stores contact titles
    EXAMPLE: 'mr. mister', 'dr. doctor'
    """
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )
    abbreviation = models.CharField (
        unique = True,
        max_length = 10,
        verbose_name = _("Abbreviation"),
    )

    class Meta:
        verbose_name = _("Contact title")
        verbose_name_plural = _("Contact titles")
        ordering = ('name',)
        permissions = (
            ('can_list_title', 'Can list Title'),
        )

    def __unicode__(self):
        return u"%s" % self.abbreviation


class Group (StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Contact group")
        verbose_name_plural = _("Contact groups")
        ordering = ('name',)
        permissions = (
            ('can_list_group', 'Can list Group'),
        )

    def __unicode__(self):
        return u"%s" % self.name

class Category (StatusModel, OrderedModel, DateModel, UserModel):
    """
    Example: cliente diretto, collocatore, prospect, istituzionale, PF, consulente
    """
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Contact category")
        verbose_name_plural = _("Contact categories")
        ordering = ('name',)
        permissions = (
            ('can_list_category', 'Can list Category'),
        )

    def __unicode__(self):
        return u"%s" % self.name

class Interest (StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Contact interest")
        verbose_name_plural = _("Contact interests")
        ordering = ('name',)
        permissions = (
            ('can_list_interest', 'Can list Interest'),
        )

    def __unicode__(self):
        return u"%s" % self.name

class Product (StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Contact product")
        verbose_name_plural = _("Contact products")
        ordering = ('name',)
        permissions = (
            ('can_list_product', 'Can list Product'),
        )

    def __unicode__(self):
        return u"%s" % self.name

class Network (StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Contact network")
        verbose_name_plural = _("Contact networks")
        ordering = ('name',)
        permissions = (
            ('can_list_network', 'Can list Network'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Contact(StatusModel, OrderedModel, DateModel, UserModel):
    user = models.ForeignKey('auth.User', blank = True, null = True,)
    title = models.ForeignKey (
        Title,
        blank = True, null = True,
        related_name='contacts',
        verbose_name = _("Title"),
    )
    first_name = models.CharField (
        max_length = 100,
        verbose_name = _("First Name"),
    )
    last_name = models.CharField (
        max_length = 100,
        verbose_name = _("Last Name"),
    )
    birth_date = models.DateField (
        null=True, blank=True,
        verbose_name = _("Birth Date"),
    )
    ssn = models.CharField (
        max_length = 16,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("SSN"),
    )
    organization = models.CharField (
        max_length = 100,
        null=True, blank=True,
        verbose_name = _("Organization"),
    )
    role = models.ForeignKey (
        Role,
        blank = True, null = True,
        related_name='contacts',
        verbose_name = _("Role"),
    )
    url = models.URLField (
        max_length=100,
        blank = True, null = True,
        verbose_name = _("URL"),
    )
    source = models.CharField (
        max_length = 100,
        null=True, blank=True,
        verbose_name = _("Source"),
    )
    category = models.ForeignKey (
        Category,
        blank = True, null = True,
        related_name='contacts',
        verbose_name = _("Category"),
    )
    groups = models.ManyToManyField (
        Group,
        related_name = "contacts",
        verbose_name = _("Groups"),
    )
    interests = models.ManyToManyField (
        Interest,
        blank = True, null = True,
        related_name = "contacts",
        verbose_name = _("Interests"),
    )
    products = models.ManyToManyField (
        Product,
        blank = True, null = True,
        related_name = "contacts",
        verbose_name = _("Products"),
    )
    network = models.ForeignKey (
        Network,
        blank = True, null = True,
        related_name = "contacts",
        verbose_name = _("Networks"),
    )
    # inizio newsletter
    date_subscription = models.DateTimeField(
        verbose_name=_("Date subscription"),
        blank=True, null=True
    )
    date_unsubscription = models.DateTimeField(
        verbose_name=_("Date unsubscription"),
        blank=True, null=True
    )
    # privacy
    date_last_modify_privacy = models.DateTimeField(
        verbose_name=_("Date last modify privacy"),
        blank=True, null=True
    )
    is_test_contact = models.BooleanField(
        default=False,
        verbose_name=_('test'),
    )
    # fine newsletter

    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    # campi portati da 'profilo' - start
    auth_trattamento_dati = models.BooleanField(
        _("Auth T.D."),
        help_text=_("Autorizzazione al trattamento dati"),
        default=False
    )
    auth_materiale_commerciale = models.BooleanField(
        _("Auth M.C."),
        help_text=_("Autorizzazione invio materiale commerciale"),
        default=False
    )
    auth_societa_terze = models.BooleanField(
        _("Auth S.T."),
        help_text=_("Vendita dei dati a terzi"),
        default=False
    )
    # campi portati da 'profilo' - end

    # autorizzazione preliminare per invio mail
    bonifica = models.BooleanField(
        _("Bonifica"),
        default=False
    )
    user = models.OneToOneField(
        User,
        blank=True, null=True,
        related_name="contact",
        verbose_name=_("User"),
    )

    class Meta:
        verbose_name = _("Contact")
        verbose_name_plural = _("Contacts")
        ordering = ('last_name', 'first_name')
        permissions = (
            ('can_list_contact', 'Can list Contact'),
        )

    def __unicode__(self):
        return u"%s %s" % (self.last_name, self.first_name)

    def save(self, auto_now_date_privacy=True, *args, **kwargs):
        if auto_now_date_privacy:
            self.set_date_last_modify_privacy()
        self.set_privacy_log()
        super(Contact, self).save(*args, **kwargs)

    def set_date_last_modify_privacy(self):
        # INSERIMENTO
        if not self.id:
            self.date_last_modify_privacy = datetime.datetime.now()
        # MODIFICA
        else:
            old_contact = Contact.objects.get(id=self.id)
            if (
                old_contact.auth_trattamento_dati != self.auth_trattamento_dati or
                old_contact.auth_materiale_commerciale != self.auth_materiale_commerciale or
                old_contact.auth_societa_terze != self.auth_societa_terze
            ):
                self.date_last_modify_privacy = datetime.datetime.now()

    def set_privacy_log(self):
        try:
            # INSERIMENTO
            ip_address = None
            url = None
            last_modifier = None

            request = get_request()
            if request:
                last_modifier = get_current_user().username
                url = request.get_full_path()
                ip_address = request.META.get('REMOTE_ADDR')

            if not self.id:
                try:
                    default_email = self.emails.get(default=True)
                except:
                    default_email = ""

                text_log = """---;%s;%s %s;%s;-;-;-;%s;%s;%s;%s;%s;""" % (
                    default_email,
                    self.last_name,
                    self.first_name,
                    last_modifier,
                    self.auth_trattamento_dati,
                    self.auth_materiale_commerciale,
                    self.auth_societa_terze,
                    ip_address,
                    url
                )
                privacy_log.info(text_log)
            # MODIFICA
            else:
                try:
                    default_email = self.emails.get(default=True)
                except:
                    default_email = None

                old_contact = Contact.objects.get(id=self.id)
                if (
                    old_contact.auth_trattamento_dati != self.auth_trattamento_dati or
                    old_contact.auth_materiale_commerciale != self.auth_materiale_commerciale or
                    old_contact.auth_societa_terze != self.auth_societa_terze
                ):
                    text_log = "%s;%s;%s %s;%s;%s;%s;%s;%s;%s;%s;%s;%s;" % (
                        self.id, default_email, self.last_name, self.first_name,
                        last_modifier,
                        old_contact.auth_trattamento_dati, old_contact.auth_materiale_commerciale, old_contact.auth_societa_terze,
                        self.auth_trattamento_dati, self.auth_materiale_commerciale, self.auth_societa_terze,
                        ip_address, url
                    )
                    privacy_log.info(text_log)
        except Exception, e:
            text_log = "ATTENZIONE!!!: 'acomea.contact.models.set_privacy_log' - errore: %s - id: %s" % (e.message,self.id)
            privacy_log.error(text_log)

    def get_list_emails(self):
        list_emails = []
        for email in self.emails.filter(status=1).order_by("-default", "email"):
            list_emails.append(email.email)
        return list_emails

    def get_string_emails(self):
        return ", ".join(self.get_list_emails())
    get_string_emails.short_description = _("Emails")

    def get_default_email(self):
        default_email = self.emails.filter(status=1, default=True)
        if default_email:
            return default_email[0].email
        return ''

    def get_email1(self):
        emails = self.emails.filter(status=1)
        try:
            return emails[1]
        except:
            return ''

    def get_email2(self):
        emails = self.emails.filter(status=1)
        try:
            return emails[2]
        except:
            return ''

    def get_list_groups(self):
        list_groups = []
        for group in self.groups.filter(status=1).order_by("name"):
            list_groups.append(group.name)
        return list_groups

    def get_string_groups(self):
        return ", ".join(self.get_list_groups())
    get_string_groups.short_description = _("Groups")

    def get_list_group_ids(self):
        list_groups = []
        for group in self.groups.filter(status=1).order_by("id"):
            list_groups.append(group.id)
        return list_groups

    def get_string_group_ids(self):
        return ", ".join(str(x) for x in self.get_list_group_ids())
    get_string_group_ids.short_description = _("Group ids")

    def get_list_phones(self):
        list_phones = []
        for phone in self.phones.filter(status=1).order_by("phone"):
            list_phones.append(phone.phone)
        return list_phones

    def get_string_phones(self):
        return ", ".join(self.get_list_phones())
    get_string_phones.short_description = _("Phones")

    def get_list_interests(self):
        list_interests = []
        for interest in self.interests.filter(status=1).order_by("name"):
            list_interests.append(interest.name)
        return list_interests

    def get_string_interests(self):
        return ", ".join(self.get_list_interests())
    get_string_interests.short_description = _("Interests")

    def get_list_interest_ids(self):
        list_interests = []
        for interest in self.interests.filter(status=1).order_by("id"):
            list_interests.append(interest.id)
        return list_interests

    def get_string_interest_ids(self):
        return ", ".join(str(x) for x in self.get_list_interest_ids())
    get_string_interest_ids.short_description = _("List interest ids")

    def get_list_products(self):
        list_products = []
        for product in self.products.filter(status=1).order_by("name"):
            list_products.append(product.name)
        return list_products

    def get_string_products(self):
        return ", ".join(self.get_list_products())
    get_string_products.short_description = _("Products")

    def get_list_product_ids(self):
        list_products = []
        for product in self.products.filter(status=1).order_by("id"):
            list_products.append(product.id)
        return list_products

    def get_string_product_ids(self):
        return ", ".join(str(x) for x in self.get_list_product_ids())
    get_string_product_ids.short_description = _("Product ids")

    def get_first_address(self):
        address_type = address_name = number = zip_code = None
        locality = province = region = None
        address_scope_id = country_id = address_name = None
        number = zip_code = locality = province = region = None
        addresses = self.addresses.filter(status=1)

        if addresses:
            address = addresses[0]

            address_type = address.address_type
            address_name = address.name
            number = address.number
            zip_code = address.zip_code
            locality = address.locality
            province = address.province
            region = address.region

            if address.address_scopes.all():
                address_scope_id = address.address_scopes.all()[0].id
            if address.country:
                country_id = address.country.id

        return {
            'get_address_scope_id': address_scope_id, 'get_address_type': address_type,
            'get_address_name': address_name, 'get_address_number': number, 'get_address_zip_code': zip_code,
            'get_address_locality': locality, 'get_address_province': province,
            'get_address_region': region, 'get_address_country_id': country_id
        }

    @property
    def can_create_message(self):
        """Check if the message can be created"""
        if not self.bonifica:
            return False, _("contatto non bonificato")
        if not self.auth_trattamento_dati:
            return False, _("autorizzazione trattamento dati negata")
        if not self.auth_materiale_commerciale:
            return False, _("autorizzazione invio materiale commerciale negata")
        return True, "OK"

class Email(StatusModel, OrderedModel, DateModel, UserModel, DefaultModel):
    contact = models.ForeignKey (
        Contact,
        related_name="emails",
        verbose_name=_("Contact")
    )
    email = models.EmailField (
        max_length = 100,
        unique = True,
        verbose_name = _("Email"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    class Meta:
        verbose_name = _("Email")
        verbose_name_plural = _("Emails")
        ordering = ('email',)
        permissions = (
            ('can_list_email', 'Can list Email'),
        )

    def __unicode__(self):
        return u"%s" % self.email

class PhoneCategory (StatusModel, OrderedModel, DateModel, UserModel):
    """
    Stores phone categories
    EXAMPLE: 'casa', 'ufficio', 'azienda'
    """
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("Phone category")
        verbose_name_plural = _("Phone categories")
        ordering = ('name',)
        permissions = (
            ('can_list_phonecategory', 'Can list Phone Category'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Phone(StatusModel, OrderedModel, DateModel, UserModel, DefaultModel):
    contact = models.ForeignKey (
        Contact,
        related_name="phones",
        verbose_name=_("Contact")
    )
    phone = models.CharField (
        max_length = 20,
        blank=True,
        verbose_name = _("Phone"),
    )
    category = models.ForeignKey (
        PhoneCategory,
        related_name="phones",
        verbose_name=_("Category")
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    class Meta:
        verbose_name = _("Phone")
        verbose_name_plural = _("Phones")
        ordering = ('category__name',)
        permissions = (
            ('can_list_phone', 'Can list Phone'),
        )

    def __unicode__(self):
        return u"%s: %s" % (self.category.name, self.phone)


class CategoryProfile(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("Profile category")
        verbose_name_plural = _("Profile categories")
        ordering = ('name',)
        permissions = (
            ('can_list_categoryprofile', 'Can list Category Profile'),
        )

    def __unicode__(self):
        return unicode(self.name)

class WebProfile(StatusModel, OrderedModel, DateModel, UserModel):
    contact = models.ForeignKey (
        Contact,
        related_name="profiles",
        verbose_name=_("Contact")
    )
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )
    category = models.ForeignKey (
        CategoryProfile,
        related_name="profiles",
        verbose_name=_("Category")
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    class Meta:
        verbose_name = _("Profile")
        verbose_name_plural = _("Profiles")
        ordering = ('name',)
        permissions = (
            ('can_list_webprofile', 'Can list Web Profile'),
        )

    def __unicode__(self):
        return unicode(self.name)


#ADDRESS_TYPE_CHOICES = (
#    ('corso', _(u'Corso')),
#    ('largo', _(u'Largo')),
#    ('localita', _(u'Località')),
#    ('piazza', _(u'Piazza')),
#    ('strada', _(u'Strada/Stretto')),
#    ('traversa', _(u'Traversa')),
#    ('via', _(u'Via')),
#    ('viale', _(u'Viale')),
#    ('vicolo', _(u'Vicolo/Vico')),
#    ('circonvallazione', _(u'Circonvallazione')),
#    ('frazione', _(u'Frazione')),
#    ('rione', _(u'Rione')),
#    ('salita', _(u'Salita')),
#    ('contrada', _(u'Contrada')),
#    ('statale', _(u'Statale')),
#    ('autostrada', _(u'Autostrada')),
#    ('cascina', _(u'Cascina')),
#    ('parco', _(u'Parco')),
#    ('spalto', _(u'Spalto')),
#    ('tangenziale', _(u'Tangenziale')),
#    ('zona industriale', _(u'Zona industriale')),
#    ('borgo', _(u'Borgo')),
#    ('galleria', _(u'Galleria')),
#    ('piazzale', _(u'Piazzale')),
#    ('corte', _(u'Corte')),
#    ('centro', _(u'Centro')),
#    ('foro', _(u'Foro')),
#    ('lungo', _(u'Lungo/Lungomare')),
#    ('calle', _(u'Calle')),
#    ('riviera', _(u'Riviera')),
#    ('villaggio', _(u'Villaggio')),
#    ('passaggio', _(u'Passaggio')),
#    ('borgata', _(u'Borgata')),
#    ('lungolago', _(u'Lungolago')),
#)

class AddressScope (StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )

    class Meta:
        verbose_name = _("Address scope")
        verbose_name_plural = _("Address scopes")
        ordering = ('name',)
        permissions = (
            ('can_list_addressscope', 'Can list Address Scope'),
        )

    def __unicode__(self):
        return unicode(self.name)

class Address (StatusModel, OrderedModel, DateModel, UserModel):
    contact = models.ForeignKey (
        Contact,
        related_name="addresses",
        verbose_name=_("Contact")
    )
    address_scopes = models.ManyToManyField (
        AddressScope,
        db_index = True,
        related_name = "addresses",
        verbose_name = _("Address scopes"),
    )
    address_type = models.CharField (
        max_length = 20,
        default = 'via',
        choices = settings.CONTACT_ADDRESS_TYPE_CHOICES,
        db_index = True,
        verbose_name = _("Address type"),
        blank = True, null = True
    )
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("Name"),
    )
    number = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("Number"),
        db_column = "anumber" # number in alcuni db (come oracle) e' una parola
        # riservata, quindi mettiamo come colonna del db
        # anumber per avere la piena compatibilita
    )
    zip_code = models.CharField (
        max_length = 20,
        db_index = True,
        verbose_name = _("Zip code"),
        blank = True, null = True,
    )
    locality = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("Locality"),
        blank = True, null = True,
    )
    ## FIXME da sistemare (rifNazione, rifProvincia)
    province = models.CharField (
        max_length = 2,
        db_index = True,
        verbose_name = _("Province"),
        blank = True, null = True,
    )
    region = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("Region"),
        blank = True, null = True,
    )
    country = models.ForeignKey (
        Country,
        db_index = True,
        default = 108, # id of Italy Country
        verbose_name = _("Country"),
    )


    def get_string_scopes(self):
        list_scopes = ""
        scopes = self.address_scopes.all()
        names = [ scope.name for  scope in scopes ]
        return ", ".join(names)
    get_string_scopes.short_description = _("Scopes")

    class Meta:
        verbose_name = _("Address")
        verbose_name_plural = _("Addresses")
        permissions = (
            ('can_list_address', 'Can list Address'),
        )

    def __unicode__(self):
        return u"""
            %(address_type)s %(name)s, %(number)s - %(locality)s
            (%(province)s) - %(country)s
        """ % {
            'address_type': self.get_address_type_display(), 'name': self.name,
            'number': self.number, 'locality': self.locality, 'province': self.province,
            'country': self.country
        }

class AcomeaEmailObject(DateModel, UserModel):
    email_object = models.OneToOneField(
        EmailObject,
        related_name = "acomea_email_object"
    )
    selected_groups = models.ManyToManyField (
        Group,
        null=True,blank=True,
        related_name = "acomea_email_objects",
        verbose_name = _("Selected Groups"),
    )
    date_create_newsletter = models.DateTimeField(
        null=True,blank=True,
        db_index=True,
    )

    def get_list_groups(self):
        list_groups = []
        for group in self.selected_groups.all().order_by("name"):
            list_groups.append(group.name)
        return list_groups

    def get_string_groups(self):
        return ", ".join(self.get_list_groups())
    get_string_groups.short_description = _("Groups")

    def get_list_group_ids(self):
        list_groups = []
        for group in self.groups.all().order_by("id"):
            list_groups.append(group.id)
        return list_groups

    class Meta:
        verbose_name = _("AcomeA Email Object")
        verbose_name_plural = _("AcomeA Email Object")
        permissions = (
            ('can_list_acomeaemailobject', 'Can list Acomea Email Object'),
        )

if settings.CMS_ENABLED:
    import acomea.contact.cms.plugins
    import acomea.contact.cms.app
