#-*- coding: utf-8 -*-

import datetime
import os
import time
import logging
import unicodecsv as csv

from django.conf import settings
from django.contrib import admin, messages
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from jmb.core.admin.options import _export_data
from jmb.async.models import Task

from jmb.async.tasks import JmbJobtasticTask

from acomea.contact.extended_mailer import Mailer
from acomea.contact.models import AcomeaEmailObject
from acomea.contact.utils.generic import send_confirm_privacydata_contact
from acomea.contact.utils.mailer import send_invite_to_contact


logger = logging.getLogger('celery.task')


def export_csv_async(modeladmin, request, queryset):
    """
    action per esportazione elementi selezionati changelist_view in .csv in modo asincrono
    """
    GET = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))

    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))

    res = ExportContactTaskCsv.delay_or_fail(
        app_label='contact', model=modeladmin.model,
        pks=pks, select_across=select_across, get=GET
    )

export_csv_async.short_description = _("Export csv async")


class ExportContactTaskCsv(JmbJobtasticTask):
    task_name = _("Esportazione contatti csv")
    significant_kwargs = []
    herd_avoidance_timeout = 60
    cache_duration = 0

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        """Export contacts

        :arg model: Contact model (needed to retrieve modeladmin)
        :arg pks:   list of ids to be used in rebuilding queryset
        :arg select_across: used to understand if all records where selected
        :arg get: the request.GET, used to rebuild the queryset
        """
        logger.info('Running ExportContact con xtastic')
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.csv" % task_id))
        task = Task.objects.get(task_id=task_id)
        msg = 'exported %s records' % count
        task.message = msg
        task.url = "tasks/%s.csv" % task_id
        task.save()
        return {'message': msg, 'url': task.url}

    def create_file(self, rows, output_filename=None):
        with open(output_filename, 'wb') as writer:
            out = csv.writer(writer, delimiter=';',
                             quotechar='"', quoting=csv.QUOTE_ALL)
            out.writerows(rows)
        return output_filename


class SendNewLetterTask(JmbJobtasticTask):
    task_name = _("Send newletter")
    significant_kwargs = []
    herd_avoidance_timeout = 60
    cache_duration = 0

    def calculate_result(
            self, model=None, pks=None, select_across=None,
            get=None, req=None, newsletter=None, test=None):

        logger.info('Sending newsletter xtastic')
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        request = HttpRequest()
        request.GET = get
        request.REQUEST = req
        if select_across:
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)

        selected_groups = None
        for key, value in request.GET.lists():
            if key == 'groups__in':
                selected_groups = value
                continue
        ##########################################################################
        sent_in_test = 0
        sent = 0
        create = 0
        not_create = 0
        today = datetime.datetime.today()
        contacts = qs.filter(status=1).exclude(date_unsubscription__lte=today)
        mailer = Mailer(newsletter, test, is_newsletter=True, is_task=True, request=request)
        date_create_newsletter = datetime.datetime.now()
        count = 0
        for contact in contacts:
            # creo il messaggio da inviare
            context = dict()
            mail_created, email_object = mailer.create_message(contact, context)
            if mail_created:
                create += 1
                if selected_groups:
                    contact_groups = contact.groups.filter(id__in=selected_groups)
                    if contact_groups:
                        # creo oggetto 'acomea_email_object' x salvare gruppi selezionati
                        acomea_email_object = AcomeaEmailObject()
                        acomea_email_object.email_object=email_object
                        acomea_email_object.date_create_newsletter=date_create_newsletter
                        acomea_email_object.save()

                        for contact_group in contact_groups:
                            acomea_email_object.selected_groups.add(contact_group)

                # se test - invio il messaggio immediatamente
                if test:
                    mailer.send(email_object)
                    sent_in_test += 1
            else:
                not_create += 1
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
            count += 1
        ##########################################################################
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        msg = 'Sent newsletter %s to %s contacts ' % (newsletter, create)
        if test:
            msg += "(TEST)"
        task.message = msg
        task.save()
        return {'message': msg}


class SendNewLetter2Task(JmbJobtasticTask):
    task_name = _("Send newletter2")
    significant_kwargs = []
    herd_avoidance_timeout = 60
    cache_duration = 0

    def calculate_result(
            self, model=None, pks=None, select_across=None,
            get=None, req=None, newsletter=None, test=None):

        logger.info('Sending newsletter2 xtastic')
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        request = HttpRequest()
        request.GET = get
        request.REQUEST = req
        if select_across:
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)

        selected_groups = None
        for key, value in get.lists():
            if key == 'groups' and value:
                selected_groups = value
                continue

        ##########################################################################
        sent_in_test = 0
        sent = 0
        create = 0
        not_create = 0
        today = datetime.datetime.today()
        contacts = qs.filter(status=1).exclude(date_unsubscription__lte=today)
        mailer = Mailer(newsletter, test, is_newsletter2=True, is_task=True, request=request)
        date_create_newsletter = datetime.datetime.now()
        count = 0
        for contact in contacts:
            # creo il messaggio da inviare
            context = dict(newsletter=newsletter, subscriber=contact)
            mail_created, email_object = mailer.create_message(contact, context)
            if mail_created:
                create += 1
                if selected_groups:
                    contact_groups = contact.groups.filter(id__in=selected_groups)
                    if contact_groups:
                        # creo oggetto 'acomea_email_object' x salvare gruppi selezionati
                        acomea_email_object = AcomeaEmailObject()
                        acomea_email_object.email_object=email_object
                        acomea_email_object.date_create_newsletter=date_create_newsletter
                        acomea_email_object.save()

                        for contact_group in contact_groups:
                            acomea_email_object.selected_groups.add(contact_group)

                # se test - invio il messaggio immediatamente
                if test:
                    mailer.send(email_object)
                    sent_in_test += 1
            else:
                not_create += 1
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
            count += 1
        ##########################################################################
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        msg = 'Sent newsletter2 %s to %s contacts' % (newsletter, create)
        if test:
            msg += "(TEST)"
        task.message = msg
        task.save()
        return {'message': msg}


class SendInviteTask(JmbJobtasticTask):
    task_name = _("Send invite")
    significant_kwargs = []
    herd_avoidance_timeout = 60
    cache_duration = 0

    def calculate_result(
            self, model=None, pks=None, select_across=None,
            get=None, req=None, event=None, test=None):

        logger.info('Sending invite xtastic')
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        request = HttpRequest()
        request.GET = get
        request.REQUEST = req
        if select_across:
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)

        ##########################################################################
        today = datetime.datetime.today()
        contacts = qs.filter(status=1).exclude(date_unsubscription__lte=today)
        count = created = 0
        for contact in contacts:
            invite_sending_values = send_invite_to_contact(contact, event, test=test, request=request)
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
            created += invite_sending_values["created"]
            count += 1
        ##########################################################################
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        msg = 'Sent invite to %s contacts' % created
        task.message = msg
        task.save()
        return {'message': msg}


class SendBonificaTask(JmbJobtasticTask):
    task_name = _("Send bonifica")
    significant_kwargs = []
    herd_avoidance_timeout = 60
    cache_duration = 0

    def calculate_result(self, model=None, pks=None, select_across=None, get=None, req=None, test=None):

        logger.info('Sending bonifica xtastic')
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        request = HttpRequest()
        request.GET = get
        request.REQUEST = req
        if select_across:
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        ##########################################################################
        today = datetime.datetime.today()
        contacts = qs.filter(status=1).exclude(date_unsubscription__lte=today)
        count = 0
        for contact in contacts:
            default_email = contact.get_default_email()
            if default_email != "":
                send_confirm_privacydata_contact(contact, default_email, request)
                count += 1
        ##########################################################################
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        msg = 'Sent bonifica to %s contacts' % count
        task.message = msg
        task.save()
        return {'message': msg}