# coding: utf-8
"""Utils for importation of contacts

Note: Questo modulo implementa le specifiche dettagliate nel file errori_caricamento.pdf
basato su:

  * numero di contatti derivante dalle mail presenti nella riga
  * numero di contatti derivante dalle mail presenti nella riga
  * numero di contatti derivante danomn e e cognome
  * numero di intersezioni fra i 2 dati sopra

Nel caso di aggiornamento del contatto senza intersezione (m1/n0, n1, np)
viene aggiornto il contatto derivante da mail solo relativamente agli
attributi fk e m2m, nella convinzione che i dati originali non siano errati
ma solo incompleti.
  
"""
# -*- coding: utf-8 -*-

import os
import re
import csv
import datetime
from dateutil import parser

import xlrd
import vobject

from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from django.core.validators import email_re

from acomea.contact.models import Contact, Title, Group, Role, Category, Interest, Product, Network
from acomea.contact.models import Email, Phone, Address, AddressScope, Country

class ParseError(Exception):
    def __init__(self, msg):
        self.msg = msg


COLUMNS = (
    'title_id', 'first_name', 'last_name', 'birth_date', 'ssn', 'organization',
    'group_id', 'role_id', 'url', 'source', 'category_id', 'interest_id', 'product_id', 'network_id',
    'date_subscription', 'date_unsubscription', 'is_test_contact',
    'default_email', 'email1', 'email2',
    'mobile', 'home_phone', 'work_phone',
    # address values
    'address_scope_id', 'address_type', 'address_name', 'address_number', 'address_zip_code',
    'address_locality', 'address_province', 'address_region', 'address_country_id',
    'note', 'auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze', 'bonifica',
    'date_last_modify_privacy'
)

REQUIRED_COLUMNS = ('first_name', 'last_name', 'group_id')

GET_CONTACT_FIELD = ('first_name', 'last_name')
CONTACT_FIELDS = (
    'first_name', 'last_name', 'birth_date', 'ssn', 'organization',
    'url', 'source', 'date_subscription', 'date_unsubscription',
    'is_test_contact', 'note',
     'bonifica',
)
AUTH_FIELDS = (
    'auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze', 'date_last_modify_privacy'
)
FOREIGNKEY_FIELDS = (
    (Title, 'title_id', 'title'),
    (Role, 'role_id', 'role'),
    (Category, 'category_id', 'category'),
    (Network, 'network_id', 'network'),
)
MANY_TO_MANY_FIELDS = (
    (Group, 'group_id', 'groups'),
    (Interest, 'interest_id', 'interests'),
    (Product, 'product_id', 'products'),
)
EMAIL_FIELDS = (
    'default_email', 'email1', 'email2',
)
GET_ADDRESS_FIELDS = (
    'name', 'locality',
)
ADDRESS_FIELDS = (
    # FIXME: ho dovuto aggiungere address_ per rispecchiare file .xls
    'address_type', 'address_name', 'address_number', 'address_zip_code',
    'address_locality', 'address_province', 'address_region', 'address_country_id'
)
MANY_TO_MANY_ADDRESS = (
    'address_scope_id',
)
PHONE_FIEDS = (
    ('mobile', 1),
    ('home_phone', 2),
    ('work_phone', 2),  # FIXME home & work share id?
)
csv.register_dialect('edn', delimiter=';')

errors = []
inserted = 0

def _get_object(model, id):
    try:
        return model.objects.get(id=id)
    except:
        return None

def _create_dict(model):
    obj_dict = {}
    for obj in model.objects.filter(status=1):
        obj_dict[obj.id]=obj
    return obj_dict

class ContactRow():
    def __init__(self, contact_dict, num):
        self.contact_dict = self._clean_dict(contact_dict)
        self.num = num
        self.contact = None
        self.errors = []
        self.warnings = []
        self.inserted = 0
        self.updated = 0

        self.email_contacts = self.get_contacts_from_emails()
        self.name_contacts = self.get_contacts_from_name()
        case = "case_" + self.select_case()
        self._case = case
        case_method = getattr(self, case, None)
        if not case_method:
            raise NotImplementedError("Case '%s' not implemented" % case)
        try:
            # self.contact è None in caso di errori 
            self.contact = case_method()
        except ParseError, e:
            self.errors.append(e.msg)

    def _clean_dict(self, contact_dict):
        # cancello campi vuoti
        for key, value in contact_dict.copy().iteritems():
            if not value:
                contact_dict.pop(key)
        return contact_dict

    def get_contacts_from_emails(self):
        "Return a set of contacts derived from given emails"
        # verifico emails
        emails = []
        for key in EMAIL_FIELDS:
            if key in self.contact_dict:
                email = self.contact_dict[key].lower().strip()
                if not email_re.match(email):
                    continue
                emails.append(email)

        contacts = Contact.objects.all()
        self.emails = emails
        if emails:
            return set(contacts.filter(emails__email__in=emails))
        else:
            return set()

    def get_contacts_from_name(self):
        "Return a set of contacts derived from given first/last name"

        get_contact_dict = {}
        if GET_CONTACT_FIELD:
            for key in GET_CONTACT_FIELD:
                key2 = key + '__iexact'  # ricerca che ignora capital
                value = self.contact_dict.get(key, '').title().strip()
                if value:
                    get_contact_dict[key2] = value

        if get_contact_dict:
            contacts = Contact.objects.filter(**get_contact_dict)
        else:
            contacts = None
        if contacts:
            return set(contacts)
        else:
            return set()

    def select_case(self):
        """
        Select which case it's applicable
        """
        # Il criterio è basato sul numero contatti derivanti dalle email, quello derivante dal nome
        # e dalle intersezioni relative
        intersezioni = len(self.email_contacts.intersection(self.name_contacts))
        if intersezioni > 1:
            intersezioni = 'p'
        m = len(self.email_contacts)
        n = len(self.name_contacts)
        if m > 1:
            m = 'p' # p='più'
        if n > 1:
            n = 'p' # p='più'
        if m == 0:  # non esistono intersezioni
            return "m%s_n%s" % (m, n)
        return "m%s_n%s_i%s" % (m, n, intersezioni)
            
    # Tutti i casi fanno riferimento al documento
    # https://docs.google.com/spreadsheet/ccc?key=0AviDEJLjPa1RdC1POXZHal9hSDhSTlZ1bkhOT1Q3WWc#gid=0
    # m0 = nessun contatto da email, n0 nessun contatto derivante dal nome
    # m1 = 1 contatto da email, n1 1 contatto derivante dal nome
    # mp = > 1 contatto da email, np > 1 contatto derivante dal nome

    
    ### Creazioni nuovi contatti
    def case_m0_n0(self):

        return self.create_contact()
        
    case_m0_n1 = case_m0_np = case_m0_n0

    def case_m1_n0_i0(self):
        self.updated += 1
        contact = self.email_contacts.pop()
        selected_email = [e.email for e in contact.emails.all() if e.email in self.emails]
        self.warnings += ['Riga %s - Aggiorno solo contatto da email per %s (1: m1n0i0)' % (
            (self.num+1), selected_email[0])]
        return contact
        
    case_m1_n1_i0 = case_m1_np_i0 = case_m1_n0_i0 

        
    ### Sono tutti aggiornamenti
    def case_m1_n1_i1(self):

        self.updated += 1
        return self.email_contacts.intersection(self.name_contacts).pop()

    case_m1_np_i1 = case_mp_n1_i1 = case_mp_np_i1 = case_m1_n1_i1

        
    def case_mp_n0_i0(self):
        self.errors += ['Riga %s - impossibile determinare quale contatto aggiornare (2: m+n0i0)' % (self.num +1)]
        return None
        
    def case_mp_n1_i0(self):
        self.errors += ['Riga %s - impossibile determinare quale contatto aggiornare (4: m+n0i0)' % (self.num +1)]
        return None
        
    def case_mp_np_i0(self):
        self.errors += ['Riga %s - impossibile determinare quale contatto aggiornare (7: m+n+i0)' % (self.num +1)]
        return None
        
    def case_mp_np_ip(self):
        self.errors += ['Riga %s - impossibile determinare quale contatto aggiornare (8: m+n+i+)' % (self.num +1)]
        return None

        
    def create_contact(self):
        error = False
        # verifico campi obbligatori
        for required_field in REQUIRED_COLUMNS:
            if not self.contact_dict.get(required_field, None):
                msg = 'Riga n. %s - senza campo obbligatorio: %s' % (self.num+1, required_field)
                raise(ParseError(msg))

        info = self.contact_dict.copy()
        for key in self.contact_dict:
            # modifico 'True' e 'False'
            if info[key] == 'True' or info[key] == 'true':
                info[key] = True
            if info[key] == 'False' or info[key] == 'false':
                info[key] = False

            if not key in CONTACT_FIELDS:
                info.pop(key)

        contact = Contact(**info)
        contact.save()
        self.inserted += 1
        return contact

    def create_contact_auth(self):
        info = {}
        for key in AUTH_FIELDS:
            info[key] = self.contact_dict.get(key)
            # modifico 'True' e 'False'
            if info[key] == 'True' or info[key] == 'true':
                info[key] = True
            if info[key] == 'False' or info[key] == 'false':
                info[key] = False
            try:
                date_privacy = self.contact_dict.get('date_last_modify_privacy', None)
                if date_privacy and (type(date_privacy) is not datetime.datetime):
                    date_privacy = parser.parse(date_privacy)

                info['date_last_modify_privacy'] = date_privacy
                if date_privacy is None:
                    msg = "Riga n. %s - 'date_last_modify_privacy' assente" % (self.num+1)
                    self.warnings.append(msg)
                    return
                if date_privacy.date() > datetime.datetime.now().date():
                    msg = "Riga n. %s - 'date_last_modify_privacy' non valida" % (self.num+1)
                    self.warnings.append(msg)
                    return

                if self.contact.date_last_modify_privacy and date_privacy.date() < self.contact.date_last_modify_privacy.date():
                    msg = "Riga n. %s - 'date_last_modify_privacy' file di import < 'date_last_modify_privacy' DB" % (self.num+1)
                    self.warnings.append(msg)
                    return
            except TypeError, e:
                msg = "Riga n. %s - Errore importazione: %s" % (self.num+1, e.message)
                self.errors.append(msg)
                return

        for key,value in info.items():
            setattr(self.contact, key, value)
        #self.contact.bonifica = True
        self.contact.save(auto_now_date_privacy=False)

    def create_emails(self):
        for key in EMAIL_FIELDS:
            if key in self.contact_dict:
                email = self.contact_dict[key].lower().strip()
                # verifico correttezza delle email
                if not email_re.match(email):
                    msg = 'Riga n. %s - %s email non valida' % (self.num + 1, email)
                    self.errors.append(msg)
                    continue

                try:
                    email, created_email = Email.objects.get_or_create(
                        contact=self.contact, email=email)
                    
                    if key == 'default_email' and not email.default:
                        email.default = True
                        email.save()
                except Exception, e:
                    msg = "Riga n. %s - Contatto id %s : %s email gia' presente " % (
                        self.num+1, self.contact.id, email
                        )
                    self.warnings.append(msg)

    def create_foreignkey(self):
                    
        for row in FOREIGNKEY_FIELDS:
            # (Title, 'title_id', 'title'),
            model, key, fk_attr = row
            if key in self.contact_dict:
                object_id = self.contact_dict[key]
                if object_id:
                    # elimino punteggiature e prendo solo parte intera
                    object_id = int(float(object_id))
                    ris = _get_object(model, object_id)

                    # se non esiste l'oggetto passo alla riga successiva
                    if not ris:
                        msg = _('%{m}s - Riga n. %{n}s id %{o}s: %s not found') % {
                            'm':model,
                            'n':self.num+1,
                            'o':object_id,
                        }
                        self.errors.append(msg)
                        continue

                    setattr(self.contact, fk_attr, ris)
                    self.contact.save()

    def create_many_to_many(self):

        many_to_many_list = []
        for row in MANY_TO_MANY_FIELDS:
            # row = (Group, 'group_id', 'groups')
            model, key, m2m_attr = row
            if key in self.contact_dict:
                value = str(self.contact_dict[key])
                if value:
                    list_object_id = re.split('\.|,', str(value))

                    for object_id in list_object_id:
                        # elimino punteggiature e prendo solo parte intera
                        object_id = int(float(object_id))
                        if object_id != 0:
                            ris = _get_object(model, object_id)
                            if not ris:
                                msg = _('%(m)s - Riga n. %(n)s id %(o)s: %(m)s not found') % {
                                    'm':model,
                                    'n':self.num+1,
                                    'o':object_id,
                                }
                                self.errors.append(msg)
                                continue

                            getattr(self.contact, m2m_attr).add(ris)

    def create_addresses(self):
        # get or create address
        get_address_dict = {'contact': self.contact}
        for key in GET_ADDRESS_FIELDS:
            key2 = 'address_' + key
            if not key2 in self.contact_dict:
                # self.errors.append(_('Riga n. %s postal address: missing value for field "%s"') % (
                #     self.num+1, key2
                # ))
                return
            if self.contact_dict[key2]:
                get_address_dict[key] = self.contact_dict[key2]
        address, created = Address.objects.get_or_create(**get_address_dict)

        # aggiorno indirizzo
        for key in ADDRESS_FIELDS:
            if key in self.contact_dict:
                setattr(address, key, self.contact_dict[key])
        address.save()

    def create_phones(self):
        # get or create phone
        for row in PHONE_FIEDS:
            key, id_ = row
            if self.contact_dict.has_key(key):
                phone, created = Phone.objects.get_or_create(
                    phone=self.contact_dict[key],
                    contact=self.contact,
                    category_id=id_
                )

    def get_or_create_contact_OLD(self):
        get_contact_dict = {}
        if GET_CONTACT_FIELD:
            for key in GET_CONTACT_FIELD:
                get_contact_dict[key] = self.contact_dict[key].title().strip()

        # verifico emails
        emails = []
        for key in EMAIL_FIELDS:
            if self.contact_dict.has_key(key):
                email = self.contact_dict[key]
                email = email.lower().strip()
                if not email_re.match(email):
                    continue
                emails.append(email)

        # se esiste almeno un'email verifico che non esista un contatto
        # con le stesse email
        contacts = Contact.objects.all()
        if emails:
            contacts = contacts.filter(emails__email__in=emails)

        # se esistono più contatti esco con errore
        if len(contacts) > 1:
            self.errors.append(
                "Riga n. %s - diversi contatti con email inserite" % self.num+1
            )
            return

        if contacts:
            # verifico che il contatto abbia gi nome e cognome
            contacts = contacts.filter(**get_contact_dict)
            if contacts:
                contact = contacts[0]
                self.changed = 1
            else:
                self.errors.append(
                    "Riga n. %s - contatto con nome e/o cognome diverso con email inserite" % (self.num+1)
                )

                return
        else:
            contact = Contact()
            self.inserted = 1
        return contact

def create_contacts(contact_dicts, importer_name=None, verbosity=0):
    """
    Create all the contacts
    """
    errors = []
    warnings = []
    inserted = 0
    updated = 0
    discarded = 0
    debug_cases = {}

    for num, contact_dict in enumerate(contact_dicts):
        done = False

        # salto intestazione
        if num == 0:
            continue
        # get or create contact
        row = ContactRow(contact_dict, num)

        if not row.contact:
            print "Discard"
            errors.append(row.errors)
            discarded += 1
            continue

        row.create_contact_auth()
        row.create_emails()                         # get or create email
        row.create_foreignkey()
        row.create_many_to_many()
        row.create_addresses()
        row.create_phones()
        if row.errors:
            errors.append(row.errors)
        if row.warnings:
            warnings.append(row.warnings)
        inserted += row.inserted
        updated += row.updated
        case = row._case
        if verbosity == 2:
            print "DBG: %s case: %s" % (num+1, row._case)
            
        
    if verbosity >= 1:
        debug_cases[case] = [num+1] if not case in debug_cases else debug_cases[case] + [num+1]
        print "Inserted: ", inserted
        print "Updated: ", updated
        print "Discarded: ", discarded
        print "Casi trattati: %s %s" % (len(debug_cases), debug_cases)
        print "ERRORS"
        for row in errors:
            print row

        print "WARNINGS"
        for row in warnings:
            print row


    return inserted, updated, discarded, errors, warnings, debug_cases


def text_contacts_import(stream, verbosity=0):
    """
    Import contact from a plaintext file, like CSV
    """
    contacts = []
    contact_reader = csv.reader(stream, dialect='edn')

    for contact_row in contact_reader:
        try:
            contact = {}
            for i in range(len(contact_row)):
                contact[COLUMNS[i]] = contact_row[i]
            contacts.append(contact)
        except IndexError:
            continue

    return create_contacts(contacts, 'text', verbosity=verbosity)


def excel_contacts_import(stream, verbosity=0):
    """Import contacts from an Excel file
    :param verbosity: 0,1,2
    """
    contacts = []
    wb = xlrd.open_workbook(file_contents=stream.read())
    sh = wb.sheet_by_index(0)

    for row in range(sh.nrows):
        contact = {}

        for i in range(len(COLUMNS)):
            try:
                # if cell value is a date, fetch it as a tuple and convert it to a datetime object
                # https://secure.simplistix.co.uk/svn/xlrd/trunk/xlrd/doc/xlrd.html?p=4966
                if sh.cell(row, i).ctype == 3:
                    date_tuple = xlrd.xldate_as_tuple(sh.cell(row, i).value, 0)
                    value = datetime.datetime(*date_tuple)
                else:
                    value = sh.cell(row, i).value
                contact[COLUMNS[i]] = value
            except IndexError:
                break

        contacts.append(contact)

    return create_contacts(contacts, 'excel', verbosity=verbosity)


def import_dispatcher(source, type_=None, verbosity=0):
    """
    Select importer and import contacts

    :param source: filename or stream (file handler)
    :param type: xls/excel or text/csv
    :param verbosity: verbosità: 0,1,2 (crescente)
    """
    if isinstance(source, basestring):
        if not type_:
            type_ = os.path.splitext(source)[1].lstrip('.')
        source = open(source, 'r')
        
    if type_ in ('text', 'csv'):
        return text_contacts_import(source, verbosity=verbosity)
    elif type_ in ('excel', 'xls'):
        return excel_contacts_import(source, verbosity=verbosity)
    return 0

import_contacts = import_dispatcher
